import * as i0 from '@angular/core';
import { Injectable, assertInInjectionContext, inject, signal, effect } from '@angular/core';

// Default to a dummy "batch" implementation that just runs the callback
function defaultNoopBatch(callback) {
    callback();
}

function createListenerCollection() {
    let first = null;
    let last = null;
    return {
        clear() {
            first = null;
            last = null;
        },
        notify() {
            defaultNoopBatch(() => {
                let listener = first;
                while (listener) {
                    listener.callback();
                    listener = listener.next;
                }
            });
        },
        get() {
            const listeners = [];
            let listener = first;
            while (listener) {
                listeners.push(listener);
                listener = listener.next;
            }
            return listeners;
        },
        subscribe(callback) {
            let isSubscribed = true;
            const listener = (last = {
                callback,
                next: null,
                prev: last,
            });
            if (listener.prev) {
                listener.prev.next = listener;
            }
            else {
                first = listener;
            }
            return function unsubscribe() {
                if (!isSubscribed || first === null)
                    return;
                isSubscribed = false;
                if (listener.next) {
                    listener.next.prev = listener.prev;
                }
                else {
                    last = listener.prev;
                }
                if (listener.prev) {
                    listener.prev.next = listener.next;
                }
                else {
                    first = listener.next;
                }
            };
        },
    };
}
const nullListeners = {
    notify() { },
    get: () => [],
};
function createSubscription(store, parentSub) {
    let unsubscribe;
    let listeners = nullListeners;
    // Reasons to keep the subscription active
    let subscriptionsAmount = 0;
    // Is this specific subscription subscribed (or only nested ones?)
    let selfSubscribed = false;
    function addNestedSub(listener) {
        trySubscribe();
        const cleanupListener = listeners.subscribe(listener);
        // cleanup nested sub
        let removed = false;
        return () => {
            if (!removed) {
                removed = true;
                cleanupListener();
                tryUnsubscribe();
            }
        };
    }
    function notifyNestedSubs() {
        listeners.notify();
    }
    function handleChangeWrapper() {
        if (subscription.onStateChange) {
            subscription.onStateChange();
        }
    }
    function isSubscribed() {
        return selfSubscribed;
    }
    function trySubscribe() {
        subscriptionsAmount++;
        if (!unsubscribe) {
            unsubscribe = parentSub
                ? parentSub.addNestedSub(handleChangeWrapper)
                : store.subscribe(handleChangeWrapper);
            listeners = createListenerCollection();
        }
    }
    function tryUnsubscribe() {
        subscriptionsAmount--;
        if (unsubscribe && subscriptionsAmount === 0) {
            unsubscribe();
            unsubscribe = undefined;
            listeners.clear();
            listeners = nullListeners;
        }
    }
    function trySubscribeSelf() {
        if (!selfSubscribed) {
            selfSubscribed = true;
            trySubscribe();
        }
    }
    function tryUnsubscribeSelf() {
        if (selfSubscribed) {
            selfSubscribed = false;
            tryUnsubscribe();
        }
    }
    const subscription = {
        addNestedSub,
        notifyNestedSubs,
        handleChangeWrapper,
        isSubscribed,
        trySubscribe: trySubscribeSelf,
        tryUnsubscribe: tryUnsubscribeSelf,
        getListeners: () => listeners,
    };
    return subscription;
}

class ReduxProvider {
    store;
    subscription;
    ngOnDestroy() {
        this.subscription.tryUnsubscribe();
        this.subscription.onStateChange = undefined;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.3", ngImport: i0, type: ReduxProvider, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
    static ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.3", ngImport: i0, type: ReduxProvider, providedIn: null });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.3", ngImport: i0, type: ReduxProvider, decorators: [{
            type: Injectable,
            args: [{ providedIn: null }]
        }] });
// TODO: Ideally this runs in the constructor, but DI doesn't allow us to pass items to the constructor?
function createReduxProvider(store) {
    const provider = new ReduxProvider();
    provider.store = store;
    const subscription = createSubscription(store);
    provider.subscription = subscription;
    subscription.onStateChange = subscription.notifyNestedSubs;
    subscription.trySubscribe();
    return provider;
}

/**
 * Injection factory, which creates a `injectStore` injection bound to a given context.
 *
 * @returns {Function} A `injectStore` injection bound to the specified context.
 */
function createStoreInjection() {
    const injectStore = () => {
        assertInInjectionContext(injectStore);
        const context = inject(ReduxProvider);
        const { store } = context;
        return store;
    };
    Object.assign(injectStore, {
        withTypes: () => injectStore,
    });
    return injectStore;
}
/**
 * A injection to access the redux store.
 *
 * @returns {any} the redux store
 *
 * @example
 *
 * import { injectStore } from '@reduxjs/angular-redux'
 *
 * @Component({
 *   selector: 'example-component',
 *   template: `<div>{{store.getState()}}</div>`
 * })
 * export class CounterComponent {
 *   store = injectStore()
 * }
 */
const injectStore = /* #__PURE__*/ createStoreInjection();

/**
 * Injection factory, which creates a `injectDispatch` injection bound to a given context.
 *
 * @returns {Function} A `injectDispatch` injection bound to the specified context.
 */
function createDispatchInjection() {
    const injectDispatch = () => {
        assertInInjectionContext(injectDispatch);
        const store = injectStore();
        return store.dispatch;
    };
    Object.assign(injectDispatch, {
        withTypes: () => injectDispatch,
    });
    return injectDispatch;
}
/**
 * A injection to access the redux `dispatch` function.
 *
 * @returns {any|function} redux store's `dispatch` function
 *
 * @example
 *
 * import { injectDispatch } from '@reduxjs/angular-redux'
 *
 * @Component({
 *   selector: 'example-component',
 *   template: `
 *     <div>
 *       <span>{{value}}</span>
 *       <button (click)="increaseCounter()">Increase counter</button>
 *     </div>
 *   `
 * })
 * export class CounterComponent {
 *   dispatch = injectDispatch()
 *   increaseCounter = () => dispatch({ type: 'increase-counter' })
 * }
 */
const injectDispatch = /* #__PURE__*/ createDispatchInjection();

const refEquality = (a, b) => a === b;
/**
 * Injection factory, which creates a `injectSelector` injection bound to a given context.
 *
 * @returns {Function} A `injectSelector` injection bound to the specified context.
 */
function createSelectorInjection() {
    const injectSelector = (selector, equalityFnOrOptions = {}) => {
        assertInInjectionContext(injectSelector);
        const reduxContext = inject(ReduxProvider);
        const { equalityFn = refEquality } = typeof equalityFnOrOptions === 'function'
            ? { equalityFn: equalityFnOrOptions }
            : equalityFnOrOptions;
        const { store, subscription } = reduxContext;
        const selectedState = signal(selector(store.getState()));
        effect((onCleanup) => {
            const unsubscribe = subscription.addNestedSub(() => {
                const data = selector(store.getState());
                if (equalityFn(selectedState(), data)) {
                    return;
                }
                selectedState.set(data);
            });
            onCleanup(() => {
                unsubscribe();
            });
        });
        return selectedState;
    };
    Object.assign(injectSelector, {
        withTypes: () => injectSelector,
    });
    return injectSelector;
}
/**
 * A injection to access the redux store's state. This injection takes a selector function
 * as an argument. The selector is called with the store state.
 *
 * This injection takes an optional equality comparison function as the second parameter
 * that allows you to customize the way the selected state is compared to determine
 * whether the component needs to be re-rendered.
 *
 * @param {Function} selector the selector function
 * @param {Function=} equalityFn the function that will be used to determine equality
 *
 * @returns {any} the selected state
 *
 * @example
 *
 * import { injectSelector } from '@reduxjs/angular-redux'
 *
 * @Component({
 *   selector: 'counter-component',
 *   template: `<div>{{counter}}</div>`
 * })
 * export class CounterComponent {
 *   counter = injectSelector(state => state.counter)
 * }
 */
const injectSelector = /* #__PURE__*/ createSelectorInjection();

function provideRedux({ store }) {
    return {
        provide: ReduxProvider,
        useValue: createReduxProvider(store)
    };
}

function is(x, y) {
    if (x === y) {
        return x !== 0 || y !== 0 || 1 / x === 1 / y;
    }
    else {
        return x !== x && y !== y;
    }
}
function shallowEqual(objA, objB) {
    if (is(objA, objB))
        return true;
    if (typeof objA !== 'object' ||
        objA === null ||
        typeof objB !== 'object' ||
        objB === null) {
        return false;
    }
    const keysA = Object.keys(objA);
    const keysB = Object.keys(objB);
    if (keysA.length !== keysB.length)
        return false;
    for (let i = 0; i < keysA.length; i++) {
        if (!Object.prototype.hasOwnProperty.call(objB, keysA[i]) ||
            !is(objA[keysA[i]], objB[keysA[i]])) {
            return false;
        }
    }
    return true;
}

/*
 * Public API Surface of @reduxjs/angular-redux
 */

/**
 * Generated bundle index. Do not edit.
 */

export { ReduxProvider, createDispatchInjection, createReduxProvider, createSelectorInjection, createStoreInjection, injectDispatch, injectSelector, injectStore, provideRedux, shallowEqual };
//# sourceMappingURL=reduxjs-angular-redux.mjs.map
